import java.io.*;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.text.rtf.*;
import java.util.regex.*;
import java.util.LinkedList;

public class ChatFile
{
   private boolean popupErrors = true; // Display errors as JOptionPane?

   // Chat File Name
   private String fileName = null;

   // The actual contents of the file in a string
   private String fileContents = null;

   
   LinkedList reponses;

   // -------------------------------------------------------------------------
   // Name: rtfText()
   // Return: void
   // Purpose: To read actual rtf based files into the relational index (exp)
   // -------------------------------------------------------------------------
   private void rtfText()
   {
      try{
         //Reading the RTF file as data input stream
         FileInputStream fin = new FileInputStream(fileName);
         DataInputStream din = new DataInputStream(fin);

         //creating a default blank styled document
         DefaultStyledDocument styledDoc = new DefaultStyledDocument();

         //Creating a RTF Editor kit
         RTFEditorKit rtfKit = new RTFEditorKit();

         //Populating the contents in the blank styled document
         rtfKit.read( din, styledDoc, 0);

         // Getting the root document
         Document doc = styledDoc.getDefaultRootElement().getDocument();

         // Get contents of the RTF document
         fileContents = doc.getText( 0, doc.getLength() );

      } catch( Exception ex ) {
         ex.printStackTrace();
         System.exit( 0 );
      }

   } // End rtfText()

   // -------------------------------------------------------------------------
   // Name: plainText()
   // Return: void
   // Purpose: To read the chat transcript generated by the Student Tool into 
   //          a gaint string. This is so bad. Could have at least used a 
   //          StringBuffer.
   // -------------------------------------------------------------------------   
   private void plainText()
   {
      File file = new File( fileName );

      // Make sure the file exists and can be read
      if( !file.exists() || !file.canRead() ) {

         if( popupErrors ) 
            popup( "Cannot access chat file " + fileName  );
         else
            console( "Cannot access chat file " + fileName  );

         System.exit( 0 );
      }   
      
      // Create bufferd reader to access file
      BufferedReader in = null; 
      
      try {
         in = new BufferedReader( new FileReader( file ) );
      }
      catch( IOException io ) {

         if( popupErrors ) 
            popup( "Chat file " + fileName + " not found"  );
         else
            console( "Chat file " + fileName + " not found"   );

         System.exit( 0 );
      }
      
      String input = null;

      fileContents = new String();

      // read entire frequency file into memory (not good)
      while( true ) {
      
         try {
            input = in.readLine();
         }
         catch( IOException io ) {

            if( popupErrors ) 
               popup( "Error reading chat file " + fileName );
            else
               console( "Error reading chat file " + fileName );

            System.exit( 0 );
         }
         
         // end of file reached
         if( input == null )
            break;

         // Let all keywords match, not case sensitive (fix in future)
         input = input.toLowerCase();

         // Add frequency record to list with a new line (Win/Mac/Unix problems)
         fileContents = fileContents.concat( input + "\n" );
      }
      
      try {
         in.close();
      }
      catch( IOException io ) {

         if( popupErrors ) 
            popup( "Error closing chat file " + fileName );
         else
            console( "Error closing chat file " + fileName );

         System.exit( 0 );
      }      

      System.out.println( fileContents );
   }
   
   // -------------------------------------------------------------------------
   // Name: ChatFile( String, boolean )
   // Return: N/A
   // Purpose: Constructor
   // -------------------------------------------------------------------------   
   public ChatFile( String fn, boolean isPlainText )
   {
      fileName = fn;
      
      if( isPlainText == true )
         plainText();
      else
         rtfText();

      // Initialize list of reponses
      reponses = new LinkedList();
   }

   // -------------------------------------------------------------------------
   // Name: parseChatFile()
   // Return: void
   // Purpose: To actual parse the file contents of the chat transcript
   // -------------------------------------------------------------------------
   public void parseChatFile()
   {
      // Create BufferedReader
      BufferedReader contentReader = new BufferedReader( new StringReader( fileContents ) );
      
      String line = null;
      String parsedReponse[];
      String chatResponse = null;
      boolean building = false;
      int index;

      while( true )
      {

         // Read a line from the RTF file buffer
         try {
            line = contentReader.readLine();
         }
         catch( IOException io ) {

            if( popupErrors ) 
               popup( "Problem reading string buffer " + fileName  );
            else
               console( "Problem reading string buffer " + fileName  );

            System.exit( 0 );
         }

         // End of file
         if( line == null )
            break;

         try {
            // if this line is the beginning of a new chat response
            if( line.matches( "^\\[([A-Za-z]| |_)+\\]" ) ) {
                 
                 if( building == true ) {

                    // process chat                    
                    reponses.add( chatResponse );
                    
                    chatResponse = line; 
                 }
                 else {
                     building = true;
                     chatResponse = line + " ";   
                 }
            }
            // if no
            else {
               //System.out.println( "just adding to the above: " + line );
               chatResponse = chatResponse.concat( line );
            }
         }
         catch( PatternSyntaxException pse ) {

            if( popupErrors ) 
               popup( "Problem parsing chat responses " + line );
            else
               console( "Problem parsing chat responses " + line );

            System.exit( 0 );
         }

      } // End while( true )
      
      if( chatResponse != null )
         reponses.add( chatResponse );
         
      System.out.println( "\nDebug parseChatFile: " );   
      for( int i = 0; i < reponses.size(); i++ )
         System.out.println( "&" + (String)reponses.get( i ) + "&" );

   } // End chatParser()

   // -------------------------------------------------------------------------
   // Name: getNextReponse( int )
   // Return: String
   // Purpose: Get chat response @ index
   // -------------------------------------------------------------------------
   public String getNextReponse( int index )
   {
      String output = null;
      
      // bounds checking?

      output = (String) reponses.get( index );
      
      return output;
   }

   // -------------------------------------------------------------------------
   // Name: getResponseNum()
   // Return: int 
   // Purpose: Return the number of chat responses
   // -------------------------------------------------------------------------
   public int getResponseNum()
   {
      return reponses.size();
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   public String toString()
   {
      String output = new String();
      
      for( int i = 0; i < reponses.size(); i++ ) {
         output = output.concat( (String)reponses.get( i ) + "\n" );
      }

      return output;
   } 

   // -------------------------------------------------------------------------
   // Name:
   // Return: 
   // Purpose:
   // -------------------------------------------------------------------------     
   private void popup( String error )
   {
      JOptionPane.showMessageDialog( null, error,
                                           "ChatFile Class Error", 
                                           JOptionPane.ERROR_MESSAGE );
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return: 
   // Purpose:
   // -------------------------------------------------------------------------     
   private void console( String error )
   {
      System.err.println( error );
   }

} // End ChatFile class