import javax.swing.*;
import javax.swing.border.*;
import java.awt.event.*;
import java.awt.*;
import java.io.*;
import java.security.*;

public class FaceAlivePanel extends JPanel implements ActionListener
{
   // BorderLayout Panels
   private JPanel northPanel = null;
   private JPanel centerPanel = null;
   private JPanel southPanel = null;
   
   // Server Info (North)
   private JLabel serverLabel = null;
   private JTextField serverField = null;
   private JLabel portLabel = null;
   private JTextField portField = null;
   private JButton connectButton = null;
   private JButton disconnectButton = null;   

   // Face Alive Emoticon Panels (Center)
   private FaceAliveEmoticonPanel offlinePanel = null;
   private FaceAliveEmoticonPanel onlinePanel = null;
   private FaceAliveEmoticonPanel sadPanel = null;
   private FaceAliveEmoticonPanel winkingPanel = null;
   private FaceAliveEmoticonPanel cryingPanel = null;
   private FaceAliveEmoticonPanel yesPanel = null;
   private FaceAliveEmoticonPanel noPanel = null;
   private FaceAliveEmoticonPanel sleepingPanel = null;
   
   // Panel Controls (South)
   private JButton pushButton = null;
   private JButton pullButton = null;

   // Default border for this panel and all components
   private Border etched = BorderFactory.createEtchedBorder();

   // Connection to the face alive server
   private FaceAliveThread faceAliveThread = null;   

   private String systemNewline = System.getProperty("line.separator");

   
   // -------------------------------------------------------------------------
   // Purpose: Constructor
   // -------------------------------------------------------------------------
   public FaceAlivePanel() {

      // Set Panel Border
      setBorder( BorderFactory.createTitledBorder( etched, "FaceAlive" ) );
      setPreferredSize( new Dimension( 600, 400 ) );
      
      // Set layout manager
      setLayout( new BorderLayout() );

      // Set North Panel
      initNorthPanel();
      add( northPanel, BorderLayout.NORTH );
      
      // Setup Center Panel -- Do after we connect to VC
      //initCenterPanel();
      //add( centerPanel, BorderLayout.CENTER );

      // Setup South Panel
      initSouthPanel();
      add( southPanel, BorderLayout.SOUTH );

      // Refresh this panel
      repaint();

      // Make sure the image store folders exist
      File tempImageStore = new File( "images" + File.separator + "tempImageStore" );      
      if( !tempImageStore.exists() )
         tempImageStore.mkdir();

      File imageStore = new File( "images" + File.separator + "ImageStore" );      
      if( !imageStore.exists() )
         imageStore.mkdir();
   }

   // -------------------------------------------------------------------------
   // Purpose: Called by NameServerThread when Username and classroom data is known
   // -------------------------------------------------------------------------
   public void initFaceAliveCenterPanel() {
      initCenterPanel();
      add( centerPanel, BorderLayout.CENTER );
   }

   // -------------------------------------------------------------------------
   // Purpose: Create the server info panel
   // -------------------------------------------------------------------------
   private void initNorthPanel() {

      // Initialize north panel
      northPanel = new JPanel( new FlowLayout() );
      northPanel.setPreferredSize( new Dimension( 200, 60 ) );       
      northPanel.setBorder( 
            BorderFactory.createTitledBorder( etched, 
                                          "Face Alive Server Information" ) );

      // Will hold the values used to initialize serverField and portField
      String server, port;

      // Load user default values
      try {
         // Create handle to configuration file
         File configFile = new File( "config_facealive.ini" );

         // Test to make sure the file exists, and can be read
         if( !configFile.exists() || !configFile.canRead() ) {
            // File is for some reason unavailable, use defaults
            server = new String( "localhost" );
            port = new String( "7654" );
         }
         else {
            // Prepare to read the file
            BufferedReader br = new BufferedReader( 
                                    new FileReader( configFile ) );
            // Read the file
            server = br.readLine();
            port = br.readLine();

            br.close();
         }
      }
      catch( IOException io ) {
         // Something went wrong while reading the file, just use defaults
         server = new String( "localhost" );
         port = new String( "7654" );
      }
      
      // Setup server name
      serverLabel = new JLabel( "Server Name: " );
      serverField = new JTextField ( server, 24 );

      // Setup server port
      portLabel = new JLabel( "Port: " );
      portField = new JTextField( port, 6 );

      // Setup connect button
      connectButton = new JButton( "Connect" );
      connectButton.addActionListener( this );

      // Setup connect button
      disconnectButton = new JButton( "Disconnect" );
      disconnectButton.addActionListener( this );
      disconnectButton.setEnabled( false );

      // Add controls to north panel
      northPanel.add( serverLabel );
      northPanel.add( serverField );
      northPanel.add( portLabel );
      northPanel.add( portField );
      northPanel.add( connectButton );
      northPanel.add( disconnectButton );

   }

   // -------------------------------------------------------------------------
   // Purpose:
   // -------------------------------------------------------------------------
   private void initCenterPanel() {

      // Initialize center panel
      centerPanel = new JPanel( new GridLayout( 8, 1 ) );
      
      // Initialize panel components
      offlinePanel = new FaceAliveEmoticonPanel( "offline" );
      onlinePanel = new FaceAliveEmoticonPanel( "online" );
      sadPanel = new FaceAliveEmoticonPanel( "sad" );
      winkingPanel = new FaceAliveEmoticonPanel( "winking" );
      cryingPanel = new FaceAliveEmoticonPanel( "crying" );
      yesPanel = new FaceAliveEmoticonPanel( "yes" );
      noPanel = new FaceAliveEmoticonPanel( "no" );
      sleepingPanel = new FaceAliveEmoticonPanel( "sleeping" );
      
      // Add emoticon panels to this panel
      centerPanel.add( offlinePanel );    
      centerPanel.add( onlinePanel );
      centerPanel.add( yesPanel );
      centerPanel.add( noPanel );
      centerPanel.add( sadPanel );
      centerPanel.add( cryingPanel );
      centerPanel.add( winkingPanel );
      centerPanel.add( sleepingPanel );
   }

   // -------------------------------------------------------------------------
   // Purpose: Initalize push/pull buttons
   // -------------------------------------------------------------------------
   private void initSouthPanel() {

      // Initialize South Panel
      southPanel = new JPanel( new FlowLayout() );

      // Initialize buttons
      pushButton = new JButton( "Push Emoticons to Server" );
      pullButton = new JButton( "Pull Emoticons from Server" );

      // Add action listeners
      pushButton.addActionListener( this );
      pullButton.addActionListener( this );

      // Add buttons to south panel
      southPanel.add( pushButton );
      southPanel.add( pullButton );

      pushButton.setEnabled( false );
      pullButton.setEnabled( false );
   }  

   // -------------------------------------------------------------------------
   // Purpose:
   // -------------------------------------------------------------------------
   public void actionPerformed( ActionEvent e ) {

      if( e.getSource() == pushButton ) {
         handlePushButton();
      }
      else if( e.getSource() == pullButton ) {
         handlePullButton();
      }
      else if( e.getSource() == connectButton ) {
         faceAliveThread = new FaceAliveThread( serverField.getText(), portField.getText() );
         faceAliveThread.start();

         if( faceAliveThread.isConnected() == false ) {

            JOptionPane.showMessageDialog(
               null,
               "Unable to connect to the FaceAlive Server",
               "Connection Error",
               JOptionPane.ERROR_MESSAGE );

            return;
         }

         connectButton.setEnabled( false );
         disconnectButton.setEnabled( true );
         pushButton.setEnabled( true );
         pullButton.setEnabled( true );
      }
      else if( e.getSource() == disconnectButton ) {
         faceAliveThread.close();

         connectButton.setEnabled( true );
         disconnectButton.setEnabled( false );
         pushButton.setEnabled( false );
         pullButton.setEnabled( false );
      }

   }

   // -------------------------------------------------------------------------
   // Purpose: Not used
   // -------------------------------------------------------------------------
   public void updateEmoticonPanel( String emoticonType ) {

      if( emoticonType.compareTo( "offline" ) == 0 ) {
         offlinePanel.updateImage();
      } else if( emoticonType.compareTo( "online" ) == 0 ) {
         onlinePanel.updateImage();
      } else if( emoticonType.compareTo( "yes" ) == 0 ) {
         yesPanel.updateImage();
      } else if( emoticonType.compareTo( "no" ) == 0 ) {
         noPanel.updateImage();
      } else if( emoticonType.compareTo( "sad" ) == 0 ) {
         sadPanel.updateImage();
      } else if( emoticonType.compareTo( "crying" ) == 0 ) {
         cryingPanel.updateImage();
      } else if( emoticonType.compareTo( "winking" ) == 0 ) {
         winkingPanel.updateImage();
      } else if( emoticonType.compareTo( "sleeping" ) == 0 ) {
         sleepingPanel.updateImage();
      }

   }

   // -------------------------------------------------------------------------
   // Purpose: Set all Emotion Panels to the default icon
   // -------------------------------------------------------------------------
   public void setToDefaultIcon() {
      offlinePanel.setToDefaultIcon();
      onlinePanel.setToDefaultIcon();
      yesPanel.setToDefaultIcon();
      noPanel.setToDefaultIcon();
      sadPanel.setToDefaultIcon();
      cryingPanel.setToDefaultIcon();
      winkingPanel.setToDefaultIcon();
      sleepingPanel.setToDefaultIcon();
   }

   // -------------------------------------------------------------------------
   // Purpose: Set all Emotion Panels to the correct icon
   // -------------------------------------------------------------------------
   public void updateImage() {
      offlinePanel.updateImage();
      onlinePanel.updateImage();
      yesPanel.updateImage();
      noPanel.updateImage();
      sadPanel.updateImage();
      cryingPanel.updateImage();
      winkingPanel.updateImage();
      sleepingPanel.updateImage();
   }

   // -------------------------------------------------------------------------
   // Purpose:
   // -------------------------------------------------------------------------
   private void handlePushButton() {

      // Personal Data
      String firstName = Global.myFirstName.toLowerCase();
      String lastName = Global.myLastName.toLowerCase();
      String classname = Global.className.toLowerCase();

      // Default emoticons path
      String defaultPath = "images" + File.separator + 
         "DefaultEmoticons" + File.separator;

      // Image store path
      String imageStorePath = "images" + File.separator + 
         "ImageStore" + File.separator + firstName + "-" + 
         lastName + "-" + classname + "-";  

      OutgoingMessage message = new OutgoingMessage( StaticFunctions.FACE_SERVER_PUSH );
      message.append( firstName + " " + lastName );
      message.append( classname );      

      String filePath = null;
      String fileData = null;

      // offline
      filePath = offlinePanel.getPath();
      if( filePath.matches( ".*tempImageStore.*" ) == true ) {
         message.append( "offline" );
         fileData = readAndEncodeString( filePath );
         message.append( fileData );
      }

      // online
      filePath = onlinePanel.getPath();
      if( filePath.matches( ".*tempImageStore.*" ) == true ) {
         message.append( "online" );
         fileData = readAndEncodeString( filePath );
         message.append( fileData );
      } 

      // yes
      filePath = yesPanel.getPath();
      if( filePath.matches( ".*tempImageStore.*" ) == true ) {
         message.append( "yes" );
         fileData = readAndEncodeString( filePath );
         message.append( fileData );
      } 

      // no
      filePath = noPanel.getPath();
      if( filePath.matches( ".*tempImageStore.*" ) == true ) {
         message.append( "no" );
         fileData = readAndEncodeString( filePath );
         message.append( fileData );
      } 

      // sad
      filePath = sadPanel.getPath();
      if( filePath.matches( ".*tempImageStore.*" ) == true ) {
         message.append( "sad" );
         fileData = readAndEncodeString( filePath );
         message.append( fileData );
      } 

      // crying
      filePath = cryingPanel.getPath();
      if( filePath.matches( ".*tempImageStore.*" ) == true ) {
         message.append( "crying" );
         fileData = readAndEncodeString( filePath );
         message.append( fileData );      
      } 

      // winking
      filePath = winkingPanel.getPath();
      if( filePath.matches( ".*tempImageStore.*" ) == true ) {
         message.append( "winking" );
         fileData = readAndEncodeString( filePath );
         message.append( fileData );      
      } 

      // sleeping
      filePath = sleepingPanel.getPath();
      if( filePath.matches( ".*tempImageStore.*" ) == true ) {
         message.append( "sleeping" );
         fileData = readAndEncodeString( filePath );
         message.append( fileData );
      } 

      faceAliveThread.faceAliveSend( message );
   }

   // -------------------------------------------------------------------------
   // Purpose:
   // -------------------------------------------------------------------------
   private String readAndEncodeString( String filePath ) {
      FileConverter fc = new FileConverter();
      String fileData = fc.encodeBytes( fc.readFile( filePath ) );
      fileData = fileData.replaceAll( systemNewline, "%@" );

      return fileData;
   }

   // -------------------------------------------------------------------------
   // Purpose:
   // -------------------------------------------------------------------------
   private void handlePullButton() {

      String[] roster = Global.classroomPanel.getRoster();
      String[] imageDigest = new String[roster.length];
      String[] splitName = null;

      String classname = Global.className.toLowerCase();

      // Default emoticons path
      String defaultPath = "images" + File.separator + 
         "DefaultEmoticons" + File.separator;

      // Image store path
      String imageStorePath = "images" + File.separator + 
         "ImageStore" + File.separator;  

      String online = "-" + classname + "-" + "online.jpg";
      String offline = "-" + classname + "-" + "offline.jpg";
      String yes = "-" + classname + "-" + "yes.jpg";
      String no = "-" + classname + "-" + "no.jpg";
      String sad = "-" + classname + "-" + "sad.jpg";
      String winking = "-" + classname + "-" + "winking.jpg";
      String crying = "-" + classname + "-" + "crying.jpg";
      String sleeping = "-" + classname + "-" + "sleeping.jpg";

      MessageDigest md = null;
      byte[] fileDataArray = null;

      OutgoingMessage message = new OutgoingMessage( StaticFunctions.FACE_SERVER_PULL );
      message.append( classname );

      try {
         md = MessageDigest.getInstance( "MD5" );
      } catch( NoSuchAlgorithmException nsae ) {
         System.out.println( nsae );
      }

      FileConverter fc = new FileConverter();

      for( int i = 0; i < roster.length; i++ ) {

         String name = roster[i].toLowerCase();
//System.out.println( name );
         // Append current name to message
         message.append( name );

         splitName = name.split( " " );

         // Check for offline image
         File file = new File( imageStorePath + splitName[0] + "-" + splitName[1] + offline );
         if( file.exists() == true ) {
            fileDataArray = fc.readFile( imageStorePath + splitName[0] + "-" + splitName[1] + offline );
            md.update( fileDataArray );
         }  else  {
            fileDataArray = fc.readFile( defaultPath + "offline.jpg" );
            md.update( fileDataArray );
         }
         message.append( hashToString( md.digest() ) );

         // Check for online image
         file = new File( imageStorePath + splitName[0] + "-" + splitName[1] + online );
         if( file.exists() == true ) {
            fileDataArray = fc.readFile( imageStorePath + splitName[0] + "-" + splitName[1] + online );
            md.update( fileDataArray );
         }  else  {
            fileDataArray = fc.readFile( defaultPath + "online.jpg" );
            md.update( fileDataArray );
         }
         message.append( hashToString( md.digest() ) );

         // Check for yes image
         file = new File( imageStorePath + splitName[0] + "-" + splitName[1] + yes );
         if( file.exists() == true ) {
            fileDataArray = fc.readFile( imageStorePath + splitName[0] + "-" + splitName[1] + yes );
            md.update( fileDataArray );
         }  else  {
            fileDataArray = fc.readFile( defaultPath + "yes.jpg" );
            md.update( fileDataArray );
         }
         message.append( hashToString( md.digest() ) );

         // Check for no image
         file = new File( imageStorePath + splitName[0] + "-" + splitName[1] + no );
         if( file.exists() == true ) {
            fileDataArray = fc.readFile( imageStorePath + splitName[0] + "-" + splitName[1] + no );
            md.update( fileDataArray );
         }  else  {
            fileDataArray = fc.readFile( defaultPath + "no.jpg" );
            md.update( fileDataArray );
         }
         message.append( hashToString( md.digest() ) );

         // Check for sad image
         file = new File( imageStorePath + splitName[0] + "-" + splitName[1] + sad );
         if( file.exists() == true ) {
            fileDataArray = fc.readFile( imageStorePath + splitName[0] + "-" + splitName[1] + sad );
            md.update( fileDataArray );
         }  else  {
            fileDataArray = fc.readFile( defaultPath + "sad.jpg" );
            md.update( fileDataArray );
         }
         message.append( hashToString( md.digest() ) );

         // Check for crying image
         file = new File( imageStorePath + splitName[0] + "-" + splitName[1] + crying );
         if( file.exists() == true ) {
            fileDataArray = fc.readFile( imageStorePath + splitName[0] + "-" + splitName[1] + crying );
            md.update( fileDataArray );
         }  else  {
            fileDataArray = fc.readFile( defaultPath + "crying.jpg" );
            md.update( fileDataArray );
         }
         String hash = hashToString( md.digest() );
//System.out.println( "cyring hash = " + hash );
         message.append( hash );

         // Check for winking image
         file = new File( imageStorePath + splitName[0] + "-" + splitName[1] + winking );
         if( file.exists() == true ) {
            fileDataArray = fc.readFile( imageStorePath + splitName[0] + "-" + splitName[1] + winking );
            md.update( fileDataArray );
         }  else  {
            fileDataArray = fc.readFile( defaultPath + "winking.jpg" );
            md.update( fileDataArray );
         }
         hash = hashToString( md.digest() );
//System.out.println( "winking hash = " + hash );
         message.append( hash );

         // Check for sleeping image
         file = new File( imageStorePath + splitName[0] + "-" + splitName[1] + sleeping );
         if( file.exists() == true ) {
            fileDataArray = fc.readFile( imageStorePath + splitName[0] + "-" + splitName[1] + sleeping );
            md.update( fileDataArray );
         }  else  {
            fileDataArray = fc.readFile( defaultPath + "sleeping.jpg" );
            md.update( fileDataArray );
         }
         message.append( hashToString( md.digest() ) );

      }

      faceAliveThread.faceAliveSend( message );
   }

   // -------------------------------------------------------------------------
   // Purpose:
   // -------------------------------------------------------------------------
   private String hashToString( byte[] array ) {
      
      char[] hash = new char[array.length * 2];

      byte hex = (byte)0;
      int k = 0;

      for( int i = 0; i < array.length; i++ ) {

         for( int j = 0; j < 2; j++ ) {

            if( j == 0 )
               hex = (byte)((((byte)array[i] & (byte)0xf0) >> (byte)4) & (byte)0x0f);
            else
               hex = (byte)((byte)array[i] & (byte)0x0f);

            switch( hex ) {
               case 0:
               case 1:
               case 2:
               case 3:
               case 4:
               case 5:
               case 6:
               case 7:
               case 8:
               case 9:
                  hash[k++] = (char)(48 + hex);                  
                  break;
               case 10:
               case 11:
               case 12:
               case 13:
               case 14:
               case 15:
                  hash[k++] = (char)(87 + hex);
                  break;
            }
            
         }  // End inner for loop
      } // End outer for loop

      return new String( hash );
   }

}
