import javax.swing.*;
import java.util.Hashtable;
import java.util.LinkedList;
import java.io.*;
import java.util.regex.*;

public class FrequencyList
{
   boolean popupErrors = true; // Display errors as JOptionPane?

   private Hashtable hTable;
   private String file;
   
   // -------------------------------------------------------------------------
   // Name: FrequencyList( String )
   // Return: N/A
   // Purpose: Constructor
   // -------------------------------------------------------------------------
   public FrequencyList( String fileName )
   {
      file = fileName;
   
      File fFile = new File( fileName );
      
      // If file does not exist, we must create it!
      if( !fFile.exists() ) {
         
         try {
            fFile.createNewFile();
         }
         catch( IOException io ) {

            if( popupErrors ) 
               popup( "Was unable to create frequency file " + fileName );
            else
               console( "Was unable to create frequency file " + fileName );

            System.exit( 0 );
         }
         
      } // end if
      
      // Make sure the file exists and can be read
      if( !fFile.exists() || !fFile.canRead() ) {

         if( popupErrors ) 
            popup( "Cannot access frequency file " + fileName );
         else
            console( "Cannot access frequency file " + fileName );

         System.exit( 0 );
      }
      
      hTable = new Hashtable();
      
      BufferedReader in = null; 
      
      try {
         in = new BufferedReader( new FileReader( fFile ) );
      }
      catch( IOException io ) {

         if( popupErrors ) 
            popup( "Cannot access frequency file " + fileName );
         else
            console( "Cannot access frequency file " + fileName );

         System.exit( 0 );
      }
      
      String input = null;
      FrequencyRecord record = null;
      
      // read entire frequency file into memory (not good)
      while( true ) {
      
         try {
            input = in.readLine();
         }
         catch( IOException io ) {

            if( popupErrors ) 
               popup( "Cannot access frequency file " + fileName );
            else
               console( "Cannot access frequency file " + fileName );

            System.exit( 0 );
         }
         
         // end of file reached
         if( input == null )
            break;

         // Add frequency record to list
         record = new FrequencyRecord( input );
         
         hTable.put( parseKeyword( input ), record );
      }
      
      try {
         in.close();
      }
      catch( IOException io ) {
         
         if( popupErrors ) 
            popup( "Cannot close buffered reader" );
         else
            console( "Cannot close buffered reader" );

         System.exit( 0 );
      }
   } // end FrequencyList( String )

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------    
   private String parseKeyword( String record )
   {
      int index = record.indexOf( ";" );
      
      record = record.substring( 0, index );
      
      return record;
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose: If the keyword is already a record
   // -------------------------------------------------------------------------      
   public boolean isKeywordARecord( String key )
   {
      FrequencyRecord record = null;
   
      if( ( record = (FrequencyRecord)hTable.get( key ) ) == null )
         return false;
            
      return true;
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose: Add a new user to the frequencyrecord
   // -------------------------------------------------------------------------      
   public boolean addUser( String key, String user )
   {
      FrequencyRecord record = null;
   
      if( ( record =(FrequencyRecord)hTable.get( key ) ) == null )
         return false;
            
      record.addUser( user );
            
      return true;   
   }

   // -------------------------------------------------------------------------
   // Name: isUserInRecord( String, String )
   // Return: boolean
   // Purpose: Determine if user is part of the frequencyrecord key
   // -------------------------------------------------------------------------      
   public boolean isUserInRecord( String key, String user )
   {
      FrequencyRecord record = null;
   
      if( ( record = (FrequencyRecord)hTable.get( key ) ) == null )
         return false;
            
      return record.isUser( user );      
   }

   // -------------------------------------------------------------------------
   // Name: incFreq( String )
   // Return: boolean
   // Purpose: Increment the frequency of the record based on keyword
   // -------------------------------------------------------------------------      
   public boolean incFreq( String key )
   {
      FrequencyRecord record = null;
   
      if( ( record = (FrequencyRecord)hTable.get( key ) ) == null )
         return false;
            
      record.incCount();      
            
      return true;
   }

   // -------------------------------------------------------------------------
   // Name: addNewKeyword( String )
   // Return: boolean ( if the record is added )
   // Purpose: To add a new FrequencyRecord with keyword keyword
   // -------------------------------------------------------------------------   
   public boolean addNewKeyword( String key )
   {
      FrequencyRecord record = null;
   
      if( ( record = (FrequencyRecord)hTable.get( key ) ) == null ) {
         hTable.put( key, new FrequencyRecord( key, false ) );   
         return true;
      }
      
      return false;
   }
   
   // -------------------------------------------------------------------------
   // Name: close()
   // Return: void
   // Purpose: To writeout the new FrequencyRecord list
   // -------------------------------------------------------------------------   
   public void close()
   {
      File fFile = new File( file );
      
      // Remove the file that existed before, we have new values
      fFile.delete();
      
      try {
         // create the new, blank, file
         fFile.createNewFile(); 
      }
      catch( IOException io ) {
         System.err.println( "Error deleteing file" );
         System.exit( 0 );
      }
      
      PrintWriter buff = null;
      
      try {
         buff = new PrintWriter( new BufferedWriter( new FileWriter( fFile ) ) );
      }
      catch( IOException io ) {
         System.err.println( "error close" );
         System.exit( 0 );
      }
      
      LinkedList list = new LinkedList( hTable.values() );
      
      // Write out the contents
      for( int i = 0; i < list.size(); i++ )
         buff.println( ((FrequencyRecord)list.get(i)).toString() );

      // Close the file         
      buff.close();   
   }
   
   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------   
   public String toString()
   {
      String output = new String( "Frequency List:\n" );

      LinkedList list = new LinkedList( hTable.values() );
      
      for( int i = 0; i < list.size(); i++ )
         output += ((FrequencyRecord)list.get(i)).toString() + "\n";

      return output;
   } // End String toString()

   // -------------------------------------------------------------------------
   // Name:
   // Return: 
   // Purpose:
   // -------------------------------------------------------------------------     
   private void popup( String error )
   {
      JOptionPane.showMessageDialog( null, error,
                                           "FrequencyList Class Error", 
                                           JOptionPane.ERROR_MESSAGE );
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return: 
   // Purpose:
   // -------------------------------------------------------------------------     
   private void console( String error )
   {
      System.err.println( error );
   }
      
} // End class FrequencyList