import javax.swing.*;
import java.util.LinkedList;
import java.io.*;
import java.util.regex.*;

public class KeywordList
{
   boolean popupErrors = true; // Display errors as JOptionPane?

   LinkedList keywordList;
   String file;
   
   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------   
   public KeywordList( String fileName )
   {
      file = fileName;
   
      File fFile = new File( fileName );
      
      // If file does not exist, we must create it!
      if( !fFile.exists() ) {
         try {
            fFile.createNewFile();
         }
         catch( IOException io ) {
            if( popupErrors ) 
               popup( "Error creating keyword file " + fileName );
            else
               console( "Error creating keyword file " + fileName  );

            System.exit( 0 );
         }
      } // end if   

      // Make sure the file exists and can be read
      if( !fFile.exists() || !fFile.canRead() ) {
         if( popupErrors ) 
            popup( "Error accessing keyword file " + fileName );
         else
            console( "Error accessing keyword file " + fileName  );

         System.exit( 0 );
      }
      
      // Initialize frequency linked list
      keywordList = new LinkedList();
      
      BufferedReader in = null; 
      
      try {
         in = new BufferedReader( new FileReader( fFile ) );
      }
      catch( IOException io ) {
         if( popupErrors ) 
            popup( "Error reading keyword file " + fileName );
         else
            console( "Error reading keyword file " + fileName  );

         System.exit( 0 );
      }
      
      String input = null;
      
      // read entire keyword file into memory (not good)
      while( true ) {
         try {
            input = in.readLine();
         }
         catch( IOException io ) {

            if( popupErrors ) 
               popup( "Error reading keyword file " + fileName );
            else
               console( "Error reading keyword file " + fileName  );

            System.exit( 0 );
         }
         
         // end of file reached
         if( input == null )
            break;

         // Add keyword to list
         keywordList.add( input );
      } // end while loop     

      try {
         in.close();
      }
      catch( IOException io ) {

         if( popupErrors ) 
            popup( "Error closing bufferedreader" );
         else
            console( "Error closing bufferedreader"  );

         System.exit( 0 );
      }
   } // End KeywordList( String fn )

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------   
   public int getKeywordSize()
   {
      return keywordList.size();
   }

   // -------------------------------------------------------------------------
   // Name: addKeywordToList( String )
   // Return: void
   // Purpose: To add a keyword to the list of keywords that the chat response
   //          is indexed by
   // -------------------------------------------------------------------------   
   public void addKeywordToList( String keyword )
   {
      for( int i = 0; i < keywordList.size(); i++ ) {
         if( ((String)keywordList.get( i )).compareToIgnoreCase( keyword ) == 0 )
            return;
      }
   
      keywordList.add( keyword );
   } // void addKeywordToList( String )

   // -------------------------------------------------------------------------
   // Name: isKeywordInList( String )
   // Return: boolean
   // Purpose: To test whether a keyword is in the list of keywords 
   // -------------------------------------------------------------------------   
   public boolean isKeywordInList( String keyword )
   {
      for( int i = 0; i < keywordList.size(); i++ ) {
         if( ((String)keywordList.get( i )).compareToIgnoreCase( keyword ) == 0 )
            return true;
      }
   
      return false;
   } // boolean isKeywordInList( String )

   // -------------------------------------------------------------------------
   // Name: isKeywordInText( String )
   // Return: LinkedList
   // Purpose: Return a list of keywords found in the text
   // -------------------------------------------------------------------------   
   public LinkedList isKeywordInText( String text )
   {
      String keyword;
      LinkedList l = new LinkedList();
   
      for( int i = 0; i < keywordList.size(); i++ ) {
         keyword = ((String)keywordList.get( i ));
         
         System.out.println( "keyword = " + keyword + " text " + text );
         
         if( text.indexOf( keyword ) > -1 )
            l.add( keyword );
      }
      
      return l;
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------   
   public String toString()
   {
      String output = new String( "Keyword List:\n" );
      
      for( int i = 0; i < keywordList.size(); i++ ) {
         output += (String)keywordList.get( i ) + "\n";
      }

      return output;
   } // End String toString()
   
   // -------------------------------------------------------------------------
   // Name: close()
   // Return: void
   // Purpose: To write out the new keyword list, if any words were added
   // -------------------------------------------------------------------------   
   public void close()
   {
      File fFile = new File( file );
      
      fFile.delete();
      
      try {
         fFile.createNewFile(); 
      }
      catch( IOException io ) {

         if( popupErrors ) 
            popup( "Error creating file " + fFile.getName() );
         else
            console( "Error creating file " + fFile.getName()  );

         System.exit( 0 );
      }
      
      PrintWriter buff = null;
      
      try {
         buff = new PrintWriter( new BufferedWriter( new FileWriter( fFile ) ) );
      }
      catch( IOException io ) {

         if( popupErrors ) 
            popup( "Error closing file " + fFile.getName() );
         else
            console( "Error closing file " + fFile.getName()  );

         System.exit( 0 );
      }
      
      for( int i = 0; i < keywordList.size(); i++ )
         buff.println( (String)keywordList.get(i) );
         
      buff.close();   
   }   

   // -------------------------------------------------------------------------
   // Name:
   // Return: 
   // Purpose:
   // -------------------------------------------------------------------------     
   private void popup( String error )
   {
      JOptionPane.showMessageDialog( null, error,
                                           "KeywordList Class Error", 
                                           JOptionPane.ERROR_MESSAGE );
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return: 
   // Purpose:
   // -------------------------------------------------------------------------     
   private void console( String error )
   {
      System.err.println( error );
   }

} // End class KeywordList