// -------------------------------------------------------------------------
// Name: Christopher Santamaria
// Date:
//--------------------------------------------------------------------------

import java.io.*;
import java.net.*;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.event.*;
import java.awt.*;
import java.util.*;
import java.awt.geom.*;

public class NameServerThread extends Thread
{
   boolean debug = false;

   String host = null;
   InetAddress hostAddress = null;
   int port = 0;

   Socket socket = null;

   InputStreamReader isr = null;
   BufferedReader socketIn = null;

   PrintWriter socketOut;

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   public NameServerThread( )
   {
      super( "NameServer" );

      if( debug )
         System.out.println( "NameServer Thread started" );

      loadConfigurationData();

      try {
         socket = new Socket( hostAddress, port );

         isr = new InputStreamReader( socket.getInputStream() );
         socketIn = new BufferedReader( isr );

         socketOut = new PrintWriter( socket.getOutputStream(), true );
      }
      catch( Exception e ) {
         System.err.println( "Exception: Couldn't connect to Namer Server @ " + host + ":" + port );
         System.exit( 1 );
      }

      if( debug )
         System.out.println( "NameServer Thread connected" );


   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   public void run()
   {
      boolean loop = true;
      String data;
      IncomingMessage messageMsg;

      int messageType;

      try {

         while( loop )
         {
            messageMsg = new IncomingMessage( socketIn.readLine() , StaticFunctions.NAME_SERVER );

            if( debug )
               System.out.println( "Message: " + messageMsg.toString() );

            messageType = Integer.parseInt( messageMsg.getNextVariable() );

            if( debug )
               System.out.println( "MessageType : " + messageType );

            switch( messageType )
            {
               case StaticFunctions.NAME_SERVER_LOGIN:
                  handleLogin( messageMsg );
                  break;
               case StaticFunctions.NAME_SERVER_COURSELIST:
                  handleCourselist( messageMsg );
                  break;
               case StaticFunctions.NAME_SERVER_IP:
                  handleIP( messageMsg );
                  break;
               case StaticFunctions.NAME_SERVER_STUDENTLIST:
                  handleStudentList( messageMsg );
                  loop = false;
                  break;
                case StaticFunctions.NAME_SERVER_ERROR:
                  handleError( messageMsg );
                  break;
            } // End switch( messageType )

         } // End while()

      } // End try
      catch( IOException io ) {
         System.out.println( "error in loop - name server" );
      }

   } // End run()

   // -------------------------------------------------------------------------
   // Purpose:
   // -------------------------------------------------------------------------
   private void handleLogin( IncomingMessage msg ) {
      OutgoingMessage output = new OutgoingMessage( StaticFunctions.CODE_STUDENT );

      output.append( Global.myFirstName );
      output.append( Global.myLastName );
      output.append( Global.myPassword );

      nameServerSend( output );
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   private void handleCourselist( IncomingMessage msg )
   {
      OutgoingMessage output = new OutgoingMessage( );

      // Selected value from JOptionPane
      Object selectedValue = null;

      // String array representation of those courses
      String courses[] = new String[ msg.getMessageSize() - 2 ];

      for( int i = 0; i < msg.getMessageSize() - 2 ; i++ )
      {
         courses[i] = msg.getNextVariable();
      }

      while( true )
      {
          selectedValue = JOptionPane.showInputDialog(
                                null,
                                "Choose one",
                                "Input",
                                JOptionPane.INFORMATION_MESSAGE,
                                null,
                                (Object[])courses,
                                (Object)courses[0] );

          // send message course selected
          // User must select a user ( cannot hit cancel )
          if( selectedValue == null )
          {
             JOptionPane.showMessageDialog( null,
                                       "You must select a course. " +
                                        "Please try again.",
                                        "System Error",
                                        JOptionPane.ERROR_MESSAGE );
          }
          else {
             break;
          }
      } // End While

      output.append( (String)selectedValue );

      // Set Gobal classname
      Global.className = (String) selectedValue ;

      // Added for faceAlive - 3/23/05
      Global.facePanel.initFaceAliveCenterPanel();
      Global.studentMenu.updateImage();

      nameServerSend( output );

   } // End handleCourselist( Message )

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   private void handleIP( IncomingMessage msg )
   {
      OutgoingMessage output = new OutgoingMessage( );

      String ip;
      String port;
      String teacherFirst, teacherLast;

      ip = msg.getNextVariable();
      port = msg.getNextVariable();

      teacherFirst = msg.getNextVariable();
      teacherLast = msg.getNextVariable();

      Global.classroomPanel.insertTeacher( teacherFirst, teacherLast );

      nameServerSend( output );

      // create vc server socket
      Global.virtualClassroomThread = new VirtualClassroomThread( ip, port );

   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   private void handleStudentList( IncomingMessage msg )
   {
     int i = msg.index;
      
     msg.index = i;
     Global.teacherVT.generateClass( msg );

     msg.index = i;
     Global.classroomPanel.populateClassroom( msg );
     
     // --- Added by E.P. 4/05 --- //
     msg.index = i;
     Global.pushPermissionPanel.populateLists( msg );  
     
     // --- Added by C.G. 4/05 --- //
     msg.index = i;
     Global.gradePanel.setStudents(msg);

      // close socket
      try {
         socket.close();
      }
      catch( IOException io ) {
         System.err.println( "Exception: Could not close socket" );
         System.exit( 1 );
      }

      Global.virtualClassroomThread.start();
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   private void handleError( IncomingMessage msg )
   {
      String errorText = msg.getNextVariable();

      JOptionPane.showMessageDialog( null,
         errorText,
         "Server Error",
         JOptionPane.ERROR_MESSAGE );

      System.exit( 1 );
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   private void loadConfigurationData()
   {
      String data = new String();

      File file = null;
      FileReader fileReader = null;
      BufferedReader bufferedReader = null;

      // Used for url load that doesn't work
      InputStreamReader ins = null;

      // Open file for reading
      try {
         file = new File( "config_student.ini" );
         fileReader = new FileReader( file );

         bufferedReader = new BufferedReader( fileReader );
      }
      catch( Exception e ) {

         promptError( "Could not access 'config_student.ini'" );

         System.err.println( "Exception: Could not access 'config_student.ini'" );
         System.exit( 1 );
      }

      // Verify proper format of file (First Line of file)
      try {
         data = bufferedReader.readLine();
      }
      catch( IOException io ) {

         promptError( "Could not read 'config_student.ini'" );

         System.err.println( "Error: Could not read 'config_student.ini'" );
         System.exit( 1 );
      }

      if( data.compareTo( new String( "[NameServer]" ) ) < 0 ) {

         promptError( "'config_student.ini' is not formated correctly" );

         System.err.println( "Error: 'config_student.ini' is not formated correctly" );
         System.exit( 1 );
      }

      // Load host address (Second Line of file)
      try {
         data = bufferedReader.readLine();
      }
      catch( IOException io ) {

         promptError( "Could not read 'config_student.ini'" );

         System.err.println( "Error: Could not read 'config_student.ini'" );
         System.exit( 1 );
      }

      if( data.indexOf( "Host=" ) == -1 ) {

         promptError( "'config_student.ini' is not formated correctly" );

         System.err.println( "Error: 'config_student.ini' is not formated correctly" );
         System.exit( 1 );
      }

      host = data.substring( 5 );

      // get Host address
      try {
         hostAddress = InetAddress.getByName( host );
      }
      catch( Exception e ) {

         promptError( "IP address for the host ('" + host + "')could not be found" );

         System.err.println( "Error: IP address for the host ('" + host + "')could not be found" );
         System.exit( 1 );
      }

      // Load port number
      try {
         data = bufferedReader.readLine();
      }
      catch( IOException io ) {

         promptError( "Could not read 'config_student.ini'" );

         System.err.println( "Error: Could not read 'config_student.ini'" );
         System.exit( 1 );
      }

      if( data.indexOf( "Port=" ) == -1 ) {

         promptError( "'config_student.ini' is not formated correctly" );

         System.err.println( "Error: 'config_student.ini' is not formated correctly" );
         System.exit( 1 );
      }

      data = data.substring( 5 );

      port = Integer.parseInt( data );

      // Close file
      try {
         bufferedReader.close();
      }
      catch( IOException io ) {

         System.err.println( "Error: Could not close BufferedReader" );
         System.exit( 1 );
      }

      if( debug ) {
         System.out.println( "Host: " + host );
         System.out.println( "Port: " + port );
      }

   } // End loadConfigurationData()


   private void promptError( String error )
   {
      JOptionPane.showMessageDialog( null,
         error,
         "config_student.ini file error",
         JOptionPane.ERROR_MESSAGE );
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   public void nameServerSend( OutgoingMessage msg )
   {
      if( debug )
         System.out.println( "message to send: " + msg.getMessage() );

      if( debug )
         JOptionPane.showMessageDialog( null,
            msg.getMessage(),
            "Outgoing Message",
            JOptionPane.ERROR_MESSAGE );

      socketOut.println( msg.getMessage() );

      if( debug )
         System.out.println( "message sent" );
   }

} // End Class NameServerThread