import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import javax.swing.border.*;

public class PublishPanel extends JPanel implements ActionListener
{
   // Push to teacher only
   private boolean pushToTeacherOnly;

   // Push permission in general
   private boolean pushPermission;

   // The main url
   private JLabel mainLabel = null;
   private JTextField mainField = null;
   private JButton mainButton = null;

   // The audio url
   private JLabel audioLabel = null;
   private JTextField audioField = null;
   private JButton audioButton = null;

   // The video url
   private JLabel videoLabel = null;
   private JTextField videoField = null;
   private JButton videoButton = null;

   // Actions of this panel, push, clear (all)
   private JPanel buttonPanel = null;
   private JButton clearButton = null;
   private JButton pushButton = null;

   // Class used to create an etched border along the perimeter of this pane
   private Border etched = null;

   // Used to enforce layout
   private GridBagLayout gridBagLayout = null;
   private GridBagConstraints gridBagConst = null;
   private Insets inst = null;

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   public PublishPanel()
   {
      pushPermission = true;
      pushToTeacherOnly = false;

      etched = BorderFactory.createEtchedBorder();

      gridBagLayout = new GridBagLayout();
      gridBagConst = new GridBagConstraints();
      inst = new Insets( 5, 5, 5, 5 );
      gridBagConst.insets = inst;
      setLayout( gridBagLayout );

      initMainPanel();
      initAudioPanel();
      initVideoPanel();
      initButtonPanel();

      setBorder( BorderFactory.createTitledBorder( etched, "Publish" ) );
      setPreferredSize( new Dimension( 600, 200 ) );
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   public void setPushPermission( boolean b )
   {
      pushPermission = b;
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   public boolean getPushPermission()
   {
      return pushPermission;
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   public void setTeacherOnlyPermission( boolean b )
   {
      pushToTeacherOnly = b;
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   public boolean getTeacherOnlyPermission()
   {
      return pushToTeacherOnly;
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   private void addComponent( Component c, int row, int column, int width, int height )
   {
      // set gridx and gridy
      gridBagConst.gridx = column;
      gridBagConst.gridy = row;

      // set gridwidth and gridheight
      gridBagConst.gridwidth = width;
      gridBagConst.gridheight = height;

      // set constrants
      gridBagLayout.setConstraints( c, gridBagConst );
      add( c );
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   private void initMainPanel()
   {
      mainLabel = new JLabel( "Main Page: " );
      mainField = new JTextField( "http://", 30 );
      mainButton = new JButton( "Preview" );

      addComponent( new JLabel(), 2, 0, 3, 1 );
      addComponent( mainLabel, 2, 3, 2, 1 );
      addComponent( mainField, 2, 5, 8, 1 );
      addComponent( mainButton, 2, 13, 2, 1 );

      mainButton.addActionListener( this );
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   private void initAudioPanel()
   {
      audioLabel = new JLabel( "Audio: " );
      audioField = new JTextField( "http://", 30 );
      audioButton = new JButton( "Preview" );

      addComponent( new JLabel(), 3, 0, 3, 1 );
      addComponent( audioLabel, 3, 3, 2, 1 );
      addComponent( audioField, 3, 5, 8, 1 );
      addComponent( audioButton, 3, 13, 2, 1 );

      audioButton.addActionListener( this );
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   private void initVideoPanel()
   {
      videoLabel = new JLabel( "Video: " );
      videoField = new JTextField( "http://", 30 );
      videoButton = new JButton( "Preview" );

      addComponent( new JLabel(), 4, 0, 3, 1 );
      addComponent( videoLabel, 4, 3, 2, 1 );
      addComponent( videoField, 4, 5, 8, 1 );
      addComponent( videoButton, 4, 13, 2, 1 );

      videoButton.addActionListener( this );
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   private void initButtonPanel()
   {
      buttonPanel = new JPanel( new FlowLayout() );

      clearButton = new JButton( "Clear" );
      pushButton = new JButton( "Push" );

      buttonPanel.add( clearButton );
      buttonPanel.add( pushButton );

      addComponent( buttonPanel, 5, 5, 8, 1 );

      clearButton.addActionListener( this );
      pushButton.addActionListener( this );
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   public void actionPerformed( ActionEvent e )
   {
      if( e.getSource() == pushButton )
      {
         if( pushPermission == false )
         {
            JOptionPane.showMessageDialog( null,
                                           "You do not have push permission.",
                                           "Push Error",
                                           JOptionPane.ERROR_MESSAGE );
         }
         else
         {
            sendPublishMessage();
         }
      }

      if( e.getSource() == clearButton )
      {
         mainField.setText( "http://" );
         audioField.setText( "http://" );
         videoField.setText( "http://" );

		 //Remove the Preview tab
		 JTabbedPane pane = Global.tabbedPane.tabbedPane;

		 if(pane.indexOfTab("Preview Page") != -1)
			 pane.remove(pane.indexOfTab("Preview Page"));
		 Global.previewBrowser = null;
      }

      if( e.getSource() == mainButton )
      {
         if( Global.previewBrowser != null )
         {
            Global.previewBrowser.loadUrl( mainField.getText() );
         }
         else
         {
            Global.previewBrowser = new WebBrowser( );
            Global.tabbedPane.addPane( Global.previewBrowser, "Preview Page" );

			//if(Global.previewThread == null)
			//{
				Global.previewThread = new Thread( Global.previewBrowser );
				Global.previewThread.start();
			//}

            Global.previewBrowser.loadUrl( mainField.getText() );
         }

      }

      if( e.getSource() == audioButton )
      {
         if( Global.previewBrowser != null )
         {
            //Global.previewBrowser.setThePage( audioField.getText() );
            Global.previewBrowser.setThePage( audioField.getText() );
         }
         else
         {
            /*Global.previewBrowser = new WebBrowser( audioField.getText() );
            Global.tabbedPane.addPane( Global.previewBrowser, "Preivew Page" );*/

            Global.previewBrowser = new WebBrowser( );
            Global.tabbedPane.addPane( Global.previewBrowser, "Preview Page" );

            Global.previewThread = new Thread( Global.previewBrowser );
            Global.previewThread.start();

            Global.previewBrowser.loadUrl( audioField.getText() );
         }
      }

      if( e.getSource() == videoButton )
      {
         if( Global.previewBrowser != null )
         {
            //Global.previewBrowser.setThePage( videoField.getText() );
            Global.previewBrowser.setThePage( videoField.getText()  );
         }
         else
         {
            /*Global.previewBrowser = new WebBrowser( videoField.getText() );
            Global.tabbedPane.addPane( Global.previewBrowser, "Preivew Page" );*/

            Global.previewBrowser = new WebBrowser( );
            Global.tabbedPane.addPane( Global.previewBrowser, "Preview Page" );

            Global.previewThread = new Thread( Global.previewBrowser );
            Global.previewThread.start();

            Global.previewBrowser.loadUrl( videoField.getText() );
         }
      }
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   private void sendPublishMessage()
   {
      OutgoingMessage msg = new OutgoingMessage();

      msg.append( Integer.toString( StaticFunctions.COMMAND_URL ) );

     if( mainField.getText().compareTo( "" ) != 0 )
         msg.append( mainField.getText() );
      else
         msg.append( " " );

      if( audioField.getText().compareTo( "" ) != 0 )
         msg.append( audioField.getText() );
      else
         msg.append( " " );

      if( videoField.getText().compareTo( "" ) != 0 )
         msg.append( videoField.getText() );
      else
         msg.append( " " );

      if( pushToTeacherOnly == true )
         msg.append( "-1" );
      else
         msg.append( "0" );

      Global.virtualClassroomThread.virtualClassroomSend( msg );
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   public void receivePublishMessage( IncomingMessage msg )
   {
      String urlMainPage = null, urlAudioPage = null, urlVideoPage = null;

      try {
         // Check to see if push is only for teacher, if so, return
         if( Integer.parseInt( msg.getLastVariable() ) == -1 )
            return;
      }
      catch( java.lang.NumberFormatException nfe ) {
         // This had to be used cause on the first CMD_URL (which is sent from the
         // vc server) no teacher only permission variable is included, so just
         // catch this exception and continue.
      }


	// Handle Main Page URL
	if( msg.hasMoreVariables() ) {

		urlMainPage = msg.getNextVariable();

		if( !urlMainPage.equals( " " ) && !urlMainPage.equals( "" ) && urlMainPage != null ) {

			urlMainPage = urlMainPage.trim();

			if( urlMainPage != " " && urlMainPage != "" ) {
				mainField.setText( urlMainPage );

				Global.webBrowser.loadUrl( urlMainPage );
	 		}
		}	
	}

	// Handle Audio Page URL
	if( msg.hasMoreVariables() ) {

		urlAudioPage = msg.getNextVariable();

		if( !urlAudioPage.equals( " " ) && !urlAudioPage.equals( "" ) && urlAudioPage != null ) {
			urlAudioPage = urlAudioPage.trim();
			audioField.setText( urlAudioPage );

			if( Global.audioBrowser != null ) {
        	    Global.audioBrowser.loadUrl( urlAudioPage );
        	}
       		else {
				Global.audioBrowser = new WebBrowser( );
         	   	Global.tabbedPane.addPane( Global.audioBrowser, "Audio Page" );

         	   	Global.audioThread = new Thread( Global.audioBrowser );
         	   	Global.audioThread.start();

         	   	Global.audioBrowser.loadUrl( urlAudioPage );
        	}
		}
	}

	// Handle Video Page URL
	if( msg.hasMoreVariables() ) {

		urlVideoPage = msg.getNextVariable();

		if( !urlVideoPage.equals( " " ) && !urlVideoPage.equals( "" ) && urlVideoPage != null ) {
			urlVideoPage = urlVideoPage.trim();
			videoField.setText( urlVideoPage );

	        if( Global.videoBrowser != null )
         	{
         	   Global.videoBrowser.loadUrl( urlVideoPage );
         	}
         	else
         	{
         	   Global.videoBrowser = new WebBrowser( );
         	   Global.tabbedPane.addPane( Global.videoBrowser, "Video Page" );

         	   Global.videoThread = new Thread( Global.videoBrowser );
         	   Global.videoThread.start();

         	   Global.videoBrowser.loadUrl( urlVideoPage );
         	}
		}
 	}

   } // End receivePublishMessage( Message )

} // End PublishPanel Class