// -------------------------------------------------------------------------
// Name: Ed Pavlak
// Date: 3/31/05
//--------------------------------------------------------------------------

import javax.swing.*;
import javax.swing.border.*;
import java.awt.event.*;
import java.awt.*;
import java.util.*;
import java.awt.geom.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

class PushPermissionPanel extends JPanel implements ActionListener
{
	private boolean debug = false;

	// Encapsulate messageTextPane|ChatOptions|inputTextPane
	JPanel listJPanel1 = null;
	JPanel listJPanel2 = null;
	
	// JLists showing push permissions of the class
	JList permittedList;
	JList notPermittedList;

	// Encapsulate buttons
	JPanel buttonJPanel = null;
	JPanel permissionButtonJPanel = null;
	JPanel teacherButtonJPanel = null;

	// Buttons
	JButton grantButton = null;
	JButton revokeButton = null;
	JButton teacherOnlyOn = null;
	JButton teacherOnlyOff = null;

	// Used to enforce the layout policy
	GridBagLayout gridBagLayout = null;
	GridBagConstraints gridBagConst = null;
	Insets inst = null;
	
	Border etched = null;
	
	private int studentCount;
	private String[] entireClassNames = null;
	
	// Holds which students do and do not have push permission
	private Vector pushPermitted = null;
	private Vector pushNotPermitted = null;
	
	
	// -------------------------------------------------------------------------
	// Name: PushPermissionPanel()
	// Return: none (Constructor)
	// Purpose: To initialize the push permission panel.
	// -------------------------------------------------------------------------
	public PushPermissionPanel()
	{
		etched = BorderFactory.createEtchedBorder();

		// Set up layout for panel
		gridBagLayout = new GridBagLayout();
		gridBagConst = new GridBagConstraints();
		inst = new Insets( 5, 5, 5, 5 );
		gridBagConst.insets = inst;
		setLayout( gridBagLayout );

		// Initialize components in this panel
		initializeListPanel();
		initializeButtonPanel();
		
		pushPermitted = new Vector();
		pushNotPermitted = new Vector();

	} // End PushPermissionPanel()
	
	
	// -------------------------------------------------------------------------
	// Name: addComponent( Component, int, int, int, int )
	// Return: void
	// Purpose: To add c to the gridbaglayout with passed in constraints
	// -------------------------------------------------------------------------
	private void addComponent( Component c, int row, int column, int width, int height )
	{
		// set gridx and gridy
		gridBagConst.gridx = column;
		gridBagConst.gridy = row;

		// set gridwidth and gridheight
		gridBagConst.gridwidth = width;
		gridBagConst.gridheight = height;

		// set constrants
		gridBagLayout.setConstraints( c, gridBagConst );

		// Add component to layout
		add( c );
   } // End addComponent( Component c, int row, int column, int width, int height )
   
   
	// -------------------------------------------------------------------------
	// Name: initializeListPanel()
	// Return: void
	// Purpose: To create and add 2 list panels to the layout
	// -------------------------------------------------------------------------
	public void initializeListPanel()
	{
		listJPanel1 = new JPanel();
		listJPanel2 = new JPanel();
		
		listJPanel1.setBorder( BorderFactory.createTitledBorder( etched, "Allowed To Push" ) );
		listJPanel2.setBorder( BorderFactory.createTitledBorder( etched, "Not Allowed To Push" ) );
		
		permittedList = new JList ();
		permittedList.setVisibleRowCount(25);
		permittedList.setFixedCellHeight(15);
		permittedList.setFixedCellWidth(200);
		permittedList.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		listJPanel1.add(new JScrollPane(permittedList));
		
		notPermittedList = new JList();
		notPermittedList.setVisibleRowCount(25);
		notPermittedList.setFixedCellHeight(15);
		notPermittedList.setFixedCellWidth(200);
		notPermittedList.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		listJPanel2.add(new JScrollPane(notPermittedList));
		
		addComponent (listJPanel1, 0, 0, 1, 1);
		addComponent (listJPanel2, 0, 2, 1, 1);
	} // End initializeListPanel()
	
	
	// -------------------------------------------------------------------------
	// Name: initializeButtonPanel()
	// Return: void
	// Purpose: To create and add Grant Permission | Revoke Permission buttons
	// -------------------------------------------------------------------------
	public void initializeButtonPanel()
	{
		// Set up the button panel
		buttonJPanel = new JPanel( new GridLayout( 2, 1, 2, 10 ) );
		permissionButtonJPanel = new JPanel( new GridLayout( 2, 1, 2, 10 ) );
		teacherButtonJPanel = new JPanel( new GridLayout( 2, 1, 2, 10 ) );
	  
		grantButton = new JButton("<- Grant Permission");
		grantButton.addActionListener(this);
		
		revokeButton = new JButton("Revoke Permission ->");
		revokeButton.addActionListener(this);
		
		permissionButtonJPanel.add (grantButton);
		permissionButtonJPanel.add (revokeButton);
		
		teacherOnlyOn = new JButton("Teacher Push Only: On");
		teacherOnlyOn.addActionListener(this);
		
		teacherOnlyOff = new JButton("Teacher Push Only: Off");
		teacherOnlyOff.addActionListener(this);
		
		teacherButtonJPanel.add (teacherOnlyOn);
		teacherButtonJPanel.add (teacherOnlyOff);
		
		buttonJPanel.add(permissionButtonJPanel);
		buttonJPanel.add(teacherButtonJPanel);

		// Add buttonPanel to layout
		addComponent( buttonJPanel, 0, 1, 1, 1 );
		
	}  // End initializeButtonPanel()
	
	
	// -------------------------------------------------------------------------
	// Name: actionPerformed( ActionEvent e )
	// Return: void
	// Purpose: To listen for actions on the panel
	// -------------------------------------------------------------------------
	public void actionPerformed( ActionEvent e )
	{
		Object[] temp1 = null;
		String[] temp2 = null;
		
		// take push permission away from selected students
		if( e.getSource() == revokeButton )
		{
			temp1 = permittedList.getSelectedValues();
			temp2 = new String[temp1.length];
			for (int i = 0; i < temp1.length; i++) {
				temp2[i] = (String)temp1[i];
				pushNotPermitted.add(temp2[i]);
				pushPermitted.remove(temp2[i]);
				sendMessage(0, temp2[i]);
			}
		}
		// give push permission to selected students
		if( e.getSource() == grantButton )
		{
			temp1 = notPermittedList.getSelectedValues();
			temp2 = new String[temp1.length];
			for (int i = 0; i < temp1.length; i++) {
				temp2[i] = (String)temp1[i];
				pushPermitted.add(temp2[i]);
				pushNotPermitted.remove(temp2[i]);
				sendMessage(1, temp2[i]);
			}
		}
		// toggle the teacher only push permission
		if( e.getSource() == teacherOnlyOn) {
			sendTeacherOnlyMessage(1);
		}
		if( e.getSource() == teacherOnlyOff) {
			sendTeacherOnlyMessage(0);
		}
			
		// update JLists to reflect changes
		updateLists();
		
	}  // End actionPerformed( ActionEvent )
	
	
	// -------------------------------------------------------------------------
	// Name: populateLists( IncomingMessage msg )
	// Return: void
	// Purpose: Fills the permitted and not permitted lists accordingly
	// -------------------------------------------------------------------------
	public void populateLists( IncomingMessage msg )
	{
		String firstName, lastName;
		
		studentCount = ( msg.getMessageSize() - 2 ) / 2;

		entireClassNames = new String[ studentCount ];

		// Retrieve Students from the msg
		for( int i = 0; i < studentCount; i++  )
		{
			firstName = msg.getNextVariable();
			lastName = msg.getNextVariable();
			entireClassNames[i] = firstName + " " + lastName;
		}
	}  // End populateLists()
	
	
	// -------------------------------------------------------------------------
	// Name: changeStudentState( IncomingMessage msg )
	// Return:
	// Purpose: Used for update (4) and teacher_online messages (7)
	// -------------------------------------------------------------------------
	public void changeStudentState( IncomingMessage msg )
	{
		String studentFirstName, studentLastName;
		int status; 
		
		studentFirstName = msg.getFirstName();
		studentLastName = msg.getLastName();
		status = Integer.parseInt( msg.getNextVariable() );
		
		// if student goes offline remove them from the list
		if (status == StaticFunctions.OFFLINE) {
			pushPermitted.remove(studentFirstName +  " " + studentLastName);
			pushNotPermitted.remove(studentFirstName +  " " + studentLastName);
			updateLists();
		}
		else {
			if (!pushPermitted.contains(studentFirstName +  " " + studentLastName)) {
				pushPermitted.add(studentFirstName +  " " + studentLastName);
				updateLists();
			}
		}
			
		if( debug )
			System.out.println( "Student: " + studentFirstName +  " " + studentLastName + " " + status );
	} // End changeStudentState( IncomingMessage msg )

	// -------------------------------------------------------------------------
	// Name: changeTeacherState( IncomingMessage msg )
	// Return:
	// Purpose: Used for update (4) and teacher_online messages (7)
	// -------------------------------------------------------------------------
	public void changeTeacherState( IncomingMessage msg )
	{
		int status = Integer.parseInt( msg.getNextVariable() );

		//teacher.updateStatus( status );
	} // End changeTeacherState( IncomingMessage msg )
	
	
	
	// -------------------------------------------------------------------------
	// Name: updateStudentList( IncomingMessage msg )
	// Return:
	// Purpose: Used to handle message update list ( 9 )
	// -------------------------------------------------------------------------
	public void updateStudentList( IncomingMessage msg )
	{
		String studentFirstName, studentLastName;
		int status;
		
		msg.getNextVariable();

		while( msg.hasMoreVariables() )
		{
			studentFirstName = msg.getNextVariable();
			studentLastName = msg.getNextVariable();
			status = Integer.parseInt( msg.getNextVariable() );
		
			// if student goes offline remove them from the list
			if (status == StaticFunctions.OFFLINE) {
				pushPermitted.remove(studentFirstName +  " " + studentLastName);
				pushNotPermitted.remove(studentFirstName +  " " + studentLastName);
				updateLists();
			}
			else {
				if (!pushPermitted.contains(studentFirstName +  " " + studentLastName)) {
					pushPermitted.add(studentFirstName +  " " + studentLastName);
					updateLists();
				}
			}
		}
	} // End updateStudentList( IncomingMessage msg )

	// -------------------------------------------------------------------------
	// Name: updateLists()
	// Return:
	// Purpose: To update the JLists to reflect changes to the permissions
	// -------------------------------------------------------------------------
	public void updateLists()
	{
		permittedList.setListData(pushPermitted);
		notPermittedList.setListData(pushNotPermitted);
	} // End updateLists()


	// -------------------------------------------------------------------------
	// Name: sendMessage(int permission, String student)
	// Return: void
	// Purpose: To send a message to change push permission
	// -------------------------------------------------------------------------
	private void sendMessage(int permission, String student)
	{
		// parse out student first and last name
		StringTokenizer st = new StringTokenizer( student, " " );
		String[] names = new String[st.countTokens()];
		
		try {
			for( int i = 0; i < names.length; i++ )
				names[i] = st.nextToken();
		}
		catch( NoSuchElementException nse ) { }

		// Create new message
		OutgoingMessage msg = new OutgoingMessage();

		// message number
		msg.append( Integer.toString( StaticFunctions.COMMAND_PUSH ) );
		
		// student name
		msg.append( names[0] );
		msg.append( names[1] );
		
		// permission
		msg.append( Integer.toString( permission ) );
		
		System.out.println (msg.getMessage());

		// Send message to server
		Global.virtualClassroomThread.virtualClassroomSend( msg );
   } // End sendMessage(int permission, String student)


	// -------------------------------------------------------------------------
	// Name: sendTeacherOnlyMessage(int permission)
	// Return: void
	// Purpose: To send a message to change push permission
	// -------------------------------------------------------------------------
	private void sendTeacherOnlyMessage(int permission)
	{
		// Create new message
		OutgoingMessage msg = new OutgoingMessage();

		// message number
		msg.append( Integer.toString( StaticFunctions.COMMAND_ONLY_TEACHER ) );
		
		// permission
		msg.append( Integer.toString( permission ) );

		// Send message to server
		Global.virtualClassroomThread.virtualClassroomSend( msg );
   } // End sendTeacherOnlyMessage(int permission)
   
} // end PushPermissionPanel class