// -------------------------------------------------------------------------
// Name: Christopher Santamaria
// Date: 
//--------------------------------------------------------------------------

import java.io.File;
import java.util.Locale;
import javax.speech.Central;
import javax.speech.Engine;
import javax.speech.synthesis.Synthesizer;
import javax.speech.synthesis.SynthesizerModeDesc;
import javax.speech.synthesis.SynthesizerProperties;
import javax.speech.synthesis.Voice;
import com.sun.speech.freetts.util.Utilities;
import java.awt.*;
import java.awt.event.*;
import java.net.*;
import java.io.*;
import javax.swing.*;
import javax.swing.event.*;

// added
import com.sun.speech.freetts.jsapi.FreeTTSEngineCentral;
import javax.speech.EngineList;
import javax.speech.EngineCreate;

public class SpeechAgent extends Thread
{
   /* Required to run the voice synthesizer */
   SynthesizerModeDesc desc = null;
   Synthesizer synthesizer = null;

   EngineCreate creator = null;
   FreeTTSEngineCentral central = null;
   EngineList list = null;

   // This is what you want the voice to say
   String speak = null;

   // If something has been queued to be said
   boolean somethingToSay = false;

   // Control variable of the thread, if not true, exit thread
   boolean run = true;

   // True if the Synthesizer has been properly initialized
   boolean valid = true;

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   public SpeechAgent()
   {
      // Do not do anything here. Too slow, initialize variables in run, when
      // a seperate thread has begun execution.
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose: Returns the status of the class. If the class has been initialzied
   //          properly, this will be true, false otherwise.
   // -------------------------------------------------------------------------
   public boolean isOnline()
   {
      return valid;
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose: To initialize all of the requirements of the synthesizer. 
   //          It is very slow. So it has been sent to its own thread, called
   //          from run.
   // -------------------------------------------------------------------------
   private void init() 
   {
      // Create a new SynthesizerModeDesc that will match the FreeTTS Synthesizer
      SynthesizerModeDesc desc = new SynthesizerModeDesc
         ( "Unlimited domain FreeTTS Speech Synthesizer from Sun Labs",
         null,
         Locale.US,
         Boolean.FALSE, // running?
         null ); // voice

      try {
         central = new FreeTTSEngineCentral();
      }
      catch( Exception e ) {
         // This is not the correct error. I should fix this.
         noSynthesizerError();
         System.out.println( "Could not create FreeTTSEngineCentral" );
         valid = false;
         return;
      }

      list = central.createEngineList(desc);

      if (list.size() > 0) {
         creator = (EngineCreate) list.get(0);
  
         try {
            synthesizer = (Synthesizer) creator.createEngine();
         }
         catch( Exception e ) {
            // This is not the correct error. I should fix this.
            noSynthesizerError();
            System.out.println( "Could not create Speach Engine" );
            valid = false;
            return;
         }
      }

      if( synthesizer == null ) 
      {
         noSynthesizerError();
         valid = false;
         return;
      }

      try {
         // get it ready to speak
         synthesizer.allocate();
         synthesizer.resume();
      }
      catch( Exception e ) {
         // This is not the correct error. I should fix this.
         noSynthesizerError();

         e.printStackTrace();
      }
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   private void noSynthesizerError()
   {
      String message = "Can't find synthesizer.\n" +
                       "Make sure that there is a \"speech.properties\" file " +
                       "at either of these locations: \n";
      
      message += "user.home    : " + System.getProperty( "user.home" ) + "\n";
      message += "java.home/lib: " + System.getProperty( "java.home" ) + File.separator + "lib\n";
		
      System.err.println( message );

      JOptionPane.showMessageDialog( null, 
         message,
         "Speech Error",
         JOptionPane.ERROR_MESSAGE );

      // Disable audio button
      Global.chatPanel.disableAudioBox();

      // thread should exit in the run() loop, no point, failure initializing
      run = false;
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   public void say( String sayThis )
   {
      // speak the "Hello world" string
      synthesizer.speakPlainText( sayThis, null );

      try {	    
         // wait till speaking is done
         synthesizer.waitEngineState(Synthesizer.QUEUE_EMPTY);
      }
      catch( Exception e ) {
         e.printStackTrace();
      }
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   public void run()
   {
      init();

      while( run == true )
      {
         if( somethingToSay == false )
         {
            try {
               sleep( 1000 );
            }
            catch( Exception e ) {

            }
         }
         else
         {
            say( speak );
            somethingToSay = false;
         }
     } // End While
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   public void speak( String s )
   {
      if( somethingToSay == true )
         return;

      speak = s;
      somethingToSay = true;
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   public void finalize() throws Throwable
   {
      run = false;

      // clean up
      synthesizer.deallocate();
   }
}