// -------------------------------------------------------------------------
// Name: Christopher Santamaria
// Date: 
//--------------------------------------------------------------------------

import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import javax.swing.border.*;
import java.net.*;
import java.io.*;


public class StudentJLabel extends JLabel implements Comparable
{
   int type;
   int status;
   
   String firstName;
   String lastName;

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   public StudentJLabel( int t, int s, String fn, String ln )
   {
      type = t;
      status = s;
      
      firstName = fn;
      lastName = ln;

      setHorizontalTextPosition( SwingConstants.CENTER );
      setVerticalTextPosition( SwingConstants.BOTTOM );

      setText( firstName + " " + lastName );
      updateImage();
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   public void updateStatus( int s )
   {
      status = s;

      updateImage();
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   public void updateImage()
   {

      // Default emoticons path
      String defaultPath = "images" + File.separator + 
         "DefaultEmoticons" + File.separator;

      // Image store path
      String imageStorePath = "images" + File.separator + 
         "ImageStore" + File.separator + firstName.toLowerCase() + "-" + 
         lastName.toLowerCase() + "-" + Global.className.toLowerCase() + "-";

      if( status == StaticFunctions.OFFLINE )
      {
         if( fileExists( imageStorePath + "offline.jpg" ) == true ) {
            reloadImageIcon( imageStorePath + "offline.jpg" );
         } else {
            reloadImageIcon( defaultPath + "offline.jpg" );
         }  
      }
      else if( status == StaticFunctions.HAPPY ) // also serves as online
      {
         if( fileExists( imageStorePath + "online.jpg" ) == true ) {
            reloadImageIcon( imageStorePath + "online.jpg" );
         } else {
            reloadImageIcon( defaultPath + "online.jpg" );
         }
      }
      else if( status == StaticFunctions.SAD )
      {
         if( fileExists( imageStorePath + "sad.jpg" ) == true ) {
            reloadImageIcon( imageStorePath + "sad.jpg" );
         } else {
            reloadImageIcon( defaultPath + "sad.jpg" );
         }
      }
      else if( status == StaticFunctions.WINKING  )
      {
         if( fileExists( imageStorePath + "winking.jpg" ) == true ) {
            reloadImageIcon( imageStorePath + "winking.jpg" );
         } else {
            reloadImageIcon( defaultPath + "winking.jpg" );
         }
      }
      else if( status == StaticFunctions.CRYING )
      {
         if( fileExists( imageStorePath + "crying.jpg" ) == true ) {
            reloadImageIcon( imageStorePath + "crying.jpg" );
         } else {
            reloadImageIcon( defaultPath + "crying.jpg" );
         }
      }
      else if( status == StaticFunctions.YES )
      {
         if( fileExists( imageStorePath + "yes.jpg" ) == true ) {
            reloadImageIcon( imageStorePath + "yes.jpg" );
         } else {
            reloadImageIcon( defaultPath + "yes.jpg" );
         }
      }
      else if( status == StaticFunctions.NO )
      {
         if( fileExists( imageStorePath + "no.jpg" ) == true ) {
            reloadImageIcon( imageStorePath + "no.jpg" );
         } else {
            reloadImageIcon( defaultPath + "no.jpg" );
         }
      }
      else if( status == StaticFunctions.SLEEPING )
      {
         if( fileExists( imageStorePath + "sleeping.jpg" ) == true ) {
            reloadImageIcon( imageStorePath + "sleeping.jpg" );
         } else {
            reloadImageIcon( defaultPath + "sleeping.jpg" );
         }
      }

      repaint();
   }   

   // -------------------------------------------------------------------------
   // Purpose:
   // -------------------------------------------------------------------------
   private void reloadImageIcon( String fileName ) {

      // Flush old icon
      setIcon( (ImageIcon) null );

      FileConverter fc = new FileConverter();
      setIcon( new ImageIcon( fc.readFile( fileName ) ) );

      fc = null;
   }  

   // -------------------------------------------------------------------------
   // Purpose: Check whether face alive file exists
   // -------------------------------------------------------------------------
   private boolean fileExists( String filePath ) {
      File file = new File( filePath );

      return file.exists();
   }
   
   // -------------------------------------------------------------------------
   // Purpose: 
   // -------------------------------------------------------------------------
   public void setToDefaultIcon() {

      String defaultPath = "images" + File.separator + 
         "DefaultEmoticons" + File.separator;

      if( status == StaticFunctions.OFFLINE )
      {
         reloadImageIcon( defaultPath + "offline.jpg" );  
      }
      else if( status == StaticFunctions.HAPPY ) // also serves as online
      {
         reloadImageIcon( defaultPath + "online.jpg" );
      }
      else if( status == StaticFunctions.SAD )
      {
         reloadImageIcon( defaultPath + "sad.jpg" );
      }
      else if( status == StaticFunctions.WINKING  )
      {
         reloadImageIcon( defaultPath + "winking.jpg" );
      }
      else if( status == StaticFunctions.CRYING )
      {
         reloadImageIcon( defaultPath + "crying.jpg" );
      }
      else if( status == StaticFunctions.YES )
      {
         reloadImageIcon( defaultPath+ "yes.jpg" );
      }
      else if( status == StaticFunctions.NO )
      {
         reloadImageIcon( defaultPath + "no.jpg" );
      }
      else if( status == StaticFunctions.SLEEPING )
      {
         reloadImageIcon( defaultPath + "sleeping.jpg" );
      }

      repaint();
   }

   // -------------------------------------------------------------------------
   // Name: compareTo( Object )
   // Return: int
   // Purpose: To compare two user context objects.
   // -------------------------------------------------------------------------
   public int compareTo( Object object )
   {
      StudentJLabel student = (StudentJLabel) object;

      String name = firstName + " " + lastName;
      
      if( name.compareToIgnoreCase( student.getName() ) == 0 )
         return 0;
      else
         return -1;
   } // End compareTo( Object )

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   public boolean isMatch( String find )
   {
      String name = firstName + " " + lastName;
      
      if( name.compareToIgnoreCase( find ) == 0 )
         return true;
      else
         return false;
   }

   // -------------------------------------------------------------------------
   // Name:  String getName()
   // Return: String (the name of the student)
   // Purpose: To return the name of the student. Called by:
   //          ClassroomPanel.fillPrivateMessageDropDownList( String[] )
   // -------------------------------------------------------------------------
   public String getName()
   {
      return firstName + " " + lastName;
   }

   // -------------------------------------------------------------------------
   // Name: boolean getOnlineStatus()
   // Return: boolean( true if online, false, if not online)
   // Purpose:To get the status of this person. Called by:
   //          ClassroomPanel.fillPrivateMessageDropDownList( String[] )
   // -------------------------------------------------------------------------
   public boolean getOnlineStatus()
   {
      if( status == StaticFunctions.OFFLINE )
         return false;

      return true;
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   public String toString()
   {
      String retVal = null;

      retVal = "Name: " + firstName + " " + lastName + "\n";

      if( type == StaticFunctions.TEACHER )
         retVal += "Type: " + "Teacher" + "\n";
      else if( type == StaticFunctions.STUDENT )
         retVal += "Type: " + "Student" + "\n";

      if( type != StaticFunctions.OFFLINE )
         retVal += "Status: " + "Online" + "\n";
      else if( type == StaticFunctions.OFFLINE )
         retVal += "Status: " + "Offline" + "\n";

      return retVal;
   }

}
