// -------------------------------------------------------------------------
// Name: Christopher Santamaria
// Date:
//--------------------------------------------------------------------------

import java.io.*;
import java.net.*;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.event.*;
import java.awt.*;
import java.util.*;
import java.awt.geom.*;

public class VirtualClassroomThread extends Thread
{
   boolean debug = false;

   InetAddress hostAddress = null;
   int port = 0;

   Socket socket = null;

   InputStreamReader isr = null;
   BufferedReader socketIn = null;

   PrintWriter socketOut;

	OutputStreamWriter out;


   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   public VirtualClassroomThread( String hostIp, String portStr )
   {
      super( "VirtualClassroomServer" );

      if( debug )
         System.out.println( "NameServer Thread started" );

      // get Host address
      try {
         hostAddress = InetAddress.getByName( hostIp );
      }
      catch( Exception e ) {
         System.err.println( "Error: IP address for the host ('" + hostIp + "')could not be found" );
         System.exit( 1 );
      }

      port = Integer.parseInt( portStr );

      if( debug )
         System.out.println( "VirtualClassroomServer Thread started" );

      try {
         socket = new Socket( hostAddress, port );

         isr = new InputStreamReader( socket.getInputStream() );
         socketIn = new BufferedReader( isr );

         socketOut = new PrintWriter( socket.getOutputStream(), true );

//out = new OutputStreamWriter(socket.getOutputStream());
      }
      catch( Exception e ) {
         System.err.println( "Exception: Couldn't connect to VirtualClassroom Server @ " + hostIp + ":" + port );
         System.exit( 1 );
      }

      if( debug )
         System.out.println( "VirtualClassroom Thread connected" );

   } // End VirtualClassroomThread( String, String )

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   public void run()
   {
      boolean loop = true;
      String data;
      IncomingMessage messageMsg;

      int messageType;

      try {

         while( loop )
         {
            messageMsg = new IncomingMessage( socketIn.readLine(), StaticFunctions.VIRTUAL_CLASSROOM_SERVER );

            if( debug ) {
               System.out.println( "Message: " + messageMsg.toString() );
            }

            messageType = Integer.parseInt( messageMsg.getNextVariable() );

            // Set last message by
            if( messageType != StaticFunctions.COMMAND_LOGIN )
               Global.statusMenu.setLastMsgItem( messageMsg.getName() );

            switch( messageType )
            {
				case StaticFunctions.COMMAND_LOGIN:
					handleLogin( messageMsg );
					break;
				case StaticFunctions.COMMAND_URL:
					handleURL( messageMsg );
					break;
				case StaticFunctions.COMMAND_MESSAGE:
					handleMessage( messageMsg );
					break;
				case StaticFunctions.COMMAND_BOARD:
					handleBoard( messageMsg );
					break;
				case StaticFunctions.COMMAND_UPDATE:
					handleUpdate( messageMsg );
					break;
				case StaticFunctions.COMMAND_PUSH:
					handlePush( messageMsg );
					break;
				case StaticFunctions.COMMAND_QUIT:
					handleQuit( messageMsg );
					break;
				case StaticFunctions.COMMAND_TEACHER_ONLINE:
					handleTeacherOnline( messageMsg );
					break;
				case StaticFunctions.COMMAND_SYSTEM_MESSAGE:
					handleSysMessage( messageMsg );
					break;
				 case StaticFunctions.COMMAND_UPDATE_LIST:
					handleUpdateList( messageMsg );
					break;
				case StaticFunctions.COMMAND_ERROR:
					handleError( messageMsg );
					break;
				case StaticFunctions.COMMAND_MEETING:
					handleMeeting( messageMsg );
					break;
				case StaticFunctions.COMMAND_MESSAGE_TEACHER:
					handleTeacherMessage( messageMsg );
					break;
				case StaticFunctions.COMMAND_MESSAGE_PRIVATE:
					handlePrivateMessage( messageMsg );
					break;
				case StaticFunctions.COMMAND_ONLY_TEACHER:
					handleOnlyTeacher( messageMsg );
					break;
				case StaticFunctions.COMMAND_DISABLE_PRIVATE:
					handleDisablePrivate( messageMsg );
					break;
				case StaticFunctions.COMMAND_START_TEST:
					handleStartTest();
					break;
				case StaticFunctions.COMMAND_QUESTION:
					handleQuestion( messageMsg );
					break;
				case StaticFunctions.COMMAND_ANSWER:
					handleAnswer( messageMsg );
					break;
				case StaticFunctions.COMMAND_END_TEST:
					handleEndTest();
					break;

            } // End switch( messageType )

         } // End while()

      } // End try
      catch( IOException io ) {

      }

   } // End run()

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   private void handleLogin( IncomingMessage msg )
   {
      OutgoingMessage output = new OutgoingMessage( StaticFunctions.CODE_STUDENT );

      output.append( Global.myFirstName );
      output.append( Global.myLastName );
      output.append( Global.className );

      virtualClassroomSend( output );
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   private void handleURL( IncomingMessage msg )
   {
      Global.publishPanel.receivePublishMessage( msg );
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   private void handleMessage( IncomingMessage msg )
   {
      Global.chatPanel.receiveChatMessage( msg );
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   private void handleBoard( IncomingMessage msg )
   {
      Global.whiteBoardPanel.addShape( msg );
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose: Message 4
   // -------------------------------------------------------------------------
   private void handleUpdate( IncomingMessage msg )
   {
	  int i = msg.index;
      Global.classroomPanel.changeStudentState( msg );

	  Global.chatPanel.fillComoboBox( );

	  // --- Added by E.P. 4/05 --- //
	  msg.index = i;
	  Global.teacherVT.changeState(msg);
	  
	  msg.index = i;
	  Global.pushPermissionPanel.changeStudentState( msg );

	  
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   private void handlePush( IncomingMessage msg )
   {
      String fn = msg.getNextVariable();
      String ln = msg.getNextVariable();

      // If message isn't for me
      if( ( fn.compareToIgnoreCase( Global.myFirstName ) != 0 ) ||
          ( ln.compareToIgnoreCase( Global.myLastName ) != 0 ) )
         return;

      int b = Integer.parseInt( msg.getNextVariable() );
      boolean perm = false;

      if( b == 0 )
         perm = false;
      else
         perm = true;

      // Set offical push permission local variable
      Global.publishPanel.setPushPermission( perm );

      // Update status menu with new variable
      Global.statusMenu.setPushItem( Global.publishPanel.getPushPermission() );
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose: Message 6
   // -------------------------------------------------------------------------
   private void handleQuit( IncomingMessage msg )
   {
      // Not yet implemented
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   private void handleTeacherOnline( IncomingMessage msg )
   {
      Global.classroomPanel.changeTeacherState( msg );
      Global.chatPanel.fillComoboBox( );
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   private void handleSysMessage( IncomingMessage msg )
   {
      JOptionPane.showMessageDialog( null,
         msg.getNextVariable(),
         "System Message",
         JOptionPane.INFORMATION_MESSAGE );
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose: Message 9
   // -------------------------------------------------------------------------
   private void handleUpdateList( IncomingMessage msg )
   {
	  int i = msg.index;
      Global.classroomPanel.updateStudentList( msg );

	  Global.chatPanel.fillComoboBox( );

	  msg.index=i;
	  Global.teacherVT.updateStudents( msg );
	  
	  // --- Added by E.P. 4/05 --- //
	  msg.index = i;
	  Global.pushPermissionPanel.updateStudentList( msg );
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   private void handleError( IncomingMessage msg )
   {
      String error = msg.getNextVariable();

      JOptionPane.showMessageDialog( null,
         error,
         "System Message",
         JOptionPane.INFORMATION_MESSAGE );

      if( error.compareTo( "Authentication Failed!" ) == 0 )
         System.exit( 1 );
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose: Message 11
   // -------------------------------------------------------------------------
   private void handleMeeting( IncomingMessage msg )
   {
      Global.agendaPanel.recieveAgendaMemo( msg );
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose: Message 12 -- Still needs testing 7/05/2004
   // -------------------------------------------------------------------------
   private void handleTeacherMessage( IncomingMessage msg )
   {
      Global.chatPanel.receiveTeacherChatMessage( msg );
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose: Message 13 -- Still needs testing 7/05/2004
   // -------------------------------------------------------------------------
   private void handlePrivateMessage( IncomingMessage msg )
   {
      Global.chatPanel.receivePrivateChatMessage( msg );
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose: Message 14
   // -------------------------------------------------------------------------
   private void handleOnlyTeacher( IncomingMessage msg )
   {
      int b = Integer.parseInt( msg.getNextVariable() );
      boolean perm = false;

      if( b == 0 )
         perm = false;
      else
         perm = true;

      // Set push to only teacher permission local variable
      Global.publishPanel.setTeacherOnlyPermission( perm );

      // Update status menu with new variable
      Global.statusMenu.setTeacherItem( Global.publishPanel.getTeacherOnlyPermission() );

      if( debug ) {
         System.out.println( "void handleOnlyTeacher( IncomingMessage msg )" );
         System.out.println( "Teacher has enabled push to only teacher = " + perm );
         System.out.println( "Local variable set to = " +
                             Global.publishPanel.getTeacherOnlyPermission() );
      }
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose: Message 15 -- Still needs testing 7/05/2004
   // -------------------------------------------------------------------------
   private void handleDisablePrivate( IncomingMessage msg )
   {
      int b = Integer.parseInt( msg.getNextVariable() );
      boolean perm = false;

      if( b == 0 )
         perm = false;
      else
         perm = true;

      if( debug ) {
         System.out.println( "void handleDisablePrivate( IncomingMessage msg )" );
         System.out.println( "Teacher has disabled private chat = " + perm );
      }

      // Set offical private permission local variable
      Global.chatPanel.setPrivatePermission( perm );

      // Update status menu with new variable
      Global.statusMenu.setPrivateItem( Global.chatPanel.getPrivatePermission() );
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose: Message 16 -- Still needs testing 10/11/2004
   // -------------------------------------------------------------------------
   private void handleStartTest()
   {      
	  Global.teacherVT.startTest();
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose: Message 17 -- Still needs testing 10/11/2004
   // -------------------------------------------------------------------------
   private void handleQuestion( IncomingMessage msg )
   {	   
	   Global.teacherVT.receiveQuestion(msg);
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose: Message 18 -- Still needs testing 10/11/2004
   // -------------------------------------------------------------------------
   private void handleAnswer( IncomingMessage msg )
   {	   
	   Global.teacherVT.receiveAnswer(msg);
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose: Message 19 -- Still needs testing 10/11/2004
   // -------------------------------------------------------------------------
   private void handleEndTest()
   {	   
	   Global.teacherVT.endTest();
   }

   // -------------------------------------------------------------------------
   // Name:
   // Return:
   // Purpose:
   // -------------------------------------------------------------------------
   public void virtualClassroomSend( OutgoingMessage msg )
   {
      if( debug ) {
         System.out.println( "message to send: " + msg.getMessage() );

         JOptionPane.showMessageDialog( null,
            msg.getMessage(),
            "VC Outgoing Message",
            JOptionPane.ERROR_MESSAGE );
      }

      // Send out the message!
      socketOut.println( msg.getMessage() );

      // Increment activity (measure liveness)
      Global.sleepTimer.incrementActivity();

      // If we were sleeping, and now we just sent a message,
      // restore the icon to what it was prior to sleeping
      if( Global.sleepTimer.isSleeping() == true ) {
         Global.sleepTimer.setSleeping( false );
         Global.sleepTimer.restorePreviousStatus();
      }

      if( debug ) {
         System.out.println( "message sent" );
      }
   } // End virtualClassroomSend( Message )

} // End Class VirtualClassroomThread